/* ====================================================================================================================

  The copyright in this software is being made available under the License included below.
  This software may be subject to other third party and   contributor rights, including patent rights, and no such
  rights are granted under this license.

  Copyright (c) 2010, FRAUNHOFER HHI
  All rights reserved.

  Redistribution and use in source and binary forms, with or without modification, are permitted only for
  the purpose of developing standards within the Joint Collaborative Team on Video Coding and for testing and
  promoting such standards. The following conditions are required to be met:

    * Redistributions of source code must retain the above copyright notice, this list of conditions and
      the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
      the following disclaimer in the documentation and/or other materials provided with the distribution.
    * The name of FRAUNHOFER HHI
      may be used to endorse or promote products derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 * ====================================================================================================================
*/

/** \file     TDecPIPETables.cpp
    \brief    PIPE encoding tables
*/

#include "TDecPIPETables.h"


const UInt  TDecPIPETables::sm_MaxTableSize[NUM_V2V_CODERS] = 
{
   1,  8,  8, 11,    6,  6,  8, 10,    9,  8,  6,  6
};

const UInt  TDecPIPETables::sm_State2Idx[64] =
{
   0,   0,   1,   2,       2,   3,   3,   4,
   4,   4,   5,   5,       5,   6,   6,   6,
   6,   7,   7,   7,       7,   8,   8,   8,
   8,   8,   8,   8,       8,   9,   9,   9,
   9,   9,   9,   9,       9,   9,   9,   9,
  10,  10,  10,  10,      10,  10,  10,  10,
  10,  10,  10,  10,      10,  10,  10,  10,
  11,  11,  11,  11,      11,  11,  11,  11
};

const UChar TDecPIPETables::sm_StateTransition[NUM_V2V_CODERS][260] =
{
  {
       1,   2,   1,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0
  },
  {
       4,   1,   2,   0,       2,   3,   2,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       8,   5,   6,   0,       6,   7,   6,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      15,   9,  21,   0,      10,  12,  11,   0,      23,  11,  11,   0,       0,   0,   0,   0,
      13,  14,  14,   0,       0,   0,   0,   0,       0,   0,   0,   0,      26,  16,  21,   0,
      17,  20,  21,   0,      18,  19,  18,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      21,  22,  21,   0,       0,   0,   0,   0,       0,   0,   0,   0,      24,  25,  24,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      27,  33,  35,   0,      28,  30,  41,   0,
      40,  29,  41,   0,       0,   0,   0,   0,      31,  32,  31,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      34,  37,  35,   0,      35,  36,  35,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      38,  39,  38,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      41,  42,  41,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0
  },
  {
       4,   1,   2,   0,       2,   3,   2,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      10,   5,   6,   0,       7,   6,   6,   0,       0,   0,   0,   0,       8,   9,   9,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      11,  14,  16,   0,      21,  12,  24,   0,
      25,  13,  26,   0,       0,   0,   0,   0,      15,  18,  16,   0,      16,  17,  16,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      19,  20,  20,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      28,  22,  24,   0,      23,  24,  24,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      26,  27,  26,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      29,  31,  36,   0,      34,  30,  36,   0,       0,   0,   0,   0,      32,  33,  33,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      35,  36,  36,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0
  },
  {
       4,   1,   2,   0,       2,   3,   2,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       8,   5,  10,   0,       6,   7,   6,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      12,   9,  10,   0,      10,  11,  10,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      16,  13,  18,   0,      14,  15,  14,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      20,  17,  18,   0,      18,  19,  18,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      24,  21,  26,   0,      22,  23,  22,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      28,  25,  26,   0,      26,  27,  26,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      32,  29,  34,   0,      30,  31,  30,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      33,  35,  34,   0,      38,  34,  34,   0,       0,   0,   0,   0,      36,  37,  36,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      39,  40,  39,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0
  },
  {
       5,   1,   4,   0,      10,   2,   4,   0,       3,   4,   4,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      17,   6,  34,   0,      29,   7,  34,   0,       8,   9,   8,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      11,  14,  16,   0,      12,  13,  12,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      15,  16,  16,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      18,  22,  38,   0,      36,  19,  38,   0,      20,  21,  20,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      23,  26,  24,   0,      24,  25,  24,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      27,  28,  27,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      30,  33,  34,   0,      31,  32,  31,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      34,  35,  34,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      37,  40,  38,   0,      38,  39,  38,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      41,  42,  41,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0
  },
  {
       2,   1,   1,   0,       0,   0,   0,   0,       3,   6,  21,   0,      13,   4,  21,   0,
      19,   5,  21,   0,       0,   0,   0,   0,       7,  10,   9,   0,       8,   9,   9,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      11,  12,  11,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      14,  16,  23,   0,      22,  15,  23,   0,       0,   0,   0,   0,
      17,  18,  17,   0,       0,   0,   0,   0,       0,   0,   0,   0,      20,  21,  21,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      23,  24,  23,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0
  },
  {
       6,   1,   2,   0,       3,   2,   2,   0,       0,   0,   0,   0,       4,   5,   4,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      14,   7,  31,   0,       8,  11,   9,   0,
       9,  10,   9,   0,       0,   0,   0,   0,       0,   0,   0,   0,      12,  13,  12,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      15,  18,  31,   0,      25,  16,  31,   0,
      29,  17,  31,   0,       0,   0,   0,   0,      19,  22,  21,   0,      20,  21,  21,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      23,  24,  23,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      32,  26,  34,   0,      27,  28,  27,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      30,  31,  31,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      36,  33,  34,   0,      34,  35,  34,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      37,  40,  38,   0,      38,  39,  38,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      41,  42,  41,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0
  },
  {
       2,   1,   4,   0,       0,   0,   0,   0,       3,   5,   4,   0,       8,   4,   4,   0,
       0,   0,   0,   0,       6,   7,   6,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      12,   9,  29,   0,      10,  11,  10,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      15,  13,  29,   0,      19,  14,  20,   0,       0,   0,   0,   0,      22,  16,  29,   0,
      17,  18,  17,   0,       0,   0,   0,   0,       0,   0,   0,   0,      20,  21,  20,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      23,  25,  29,   0,      28,  24,  29,   0,
       0,   0,   0,   0,      26,  27,  26,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      30,  29,  29,   0,       0,   0,   0,   0,      31,  32,  31,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0
  },
  {
       2,   1,   4,   0,       0,   0,   0,   0,       6,   3,   4,   0,       4,   5,   4,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      10,   7,  27,   0,       8,   9,   8,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      11,  14,  27,   0,      24,  12,  27,   0,
      21,  13,  13,   0,       0,   0,   0,   0,      15,  18,  16,   0,      16,  17,  16,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      19,  20,  19,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      22,  23,  22,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      25,  29,  27,   0,      36,  26,  27,   0,      27,  28,  27,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      30,  33,  31,   0,      31,  32,  31,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      34,  35,  34,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      37,  40,  38,   0,      38,  39,  38,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      41,  42,  41,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0
  },
  {
       2,   1,  21,   0,       0,   0,   0,   0,       3,   7,  21,   0,      14,   4,  21,   0,
       5,   6,   5,   0,       0,   0,   0,   0,       0,   0,   0,   0,       8,  11,   9,   0,
       9,  10,   9,   0,       0,   0,   0,   0,       0,   0,   0,   0,      12,  13,  12,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      18,  15,  21,   0,      16,  17,  16,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      26,  19,  21,   0,      20,  23,  21,   0,
      21,  22,  21,   0,       0,   0,   0,   0,       0,   0,   0,   0,      24,  25,  24,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      27,  34,  29,   0,      28,  31,  29,   0,
      29,  30,  29,   0,       0,   0,   0,   0,       0,   0,   0,   0,      32,  33,  32,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      35,  38,  36,   0,      36,  37,  36,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      39,  40,  39,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0
  },
  {
       2,   1,   6,   0,       0,   0,   0,   0,       3,  17,   6,   0,       4,  10,   6,   0,
       5,   7,   6,   0,      32,   6,   6,   0,       0,   0,   0,   0,       8,   9,   8,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      11,  14,  12,   0,      12,  13,  12,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      15,  16,  15,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      18,  25,  20,   0,      19,  22,  20,   0,      20,  21,  20,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      23,  24,  23,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      26,  29,  27,   0,      27,  28,  27,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      30,  31,  30,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      33,  34,  33,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,       0,   0,   0,   0,
       0,   0,   0,   0
  },
  {
       2,   1,  64,   0,       0,   0,   0,   0,       3,  34,  64,   0,       4,  19,  33,   0,
       5,  12,  18,   0,       6,   9,  11,   0,       7,   8,   8,   0,       0,   0,   0,   0,
       0,   0,   0,   0,      10,  11,  11,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      13,  16,  18,   0,      14,  15,  15,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      17,  18,  18,   0,       0,   0,   0,   0,       0,   0,   0,   0,      20,  27,  33,   0,
      21,  24,  26,   0,      22,  23,  23,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      25,  26,  26,   0,       0,   0,   0,   0,       0,   0,   0,   0,      28,  31,  33,   0,
      29,  30,  30,   0,       0,   0,   0,   0,       0,   0,   0,   0,      32,  33,  33,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      35,  50,  64,   0,      36,  43,  49,   0,
      37,  40,  42,   0,      38,  39,  39,   0,       0,   0,   0,   0,       0,   0,   0,   0,
      41,  42,  42,   0,       0,   0,   0,   0,       0,   0,   0,   0,      44,  47,  49,   0,
      45,  46,  46,   0,       0,   0,   0,   0,       0,   0,   0,   0,      48,  49,  49,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      51,  58,  64,   0,      52,  55,  57,   0,
      53,  54,  54,   0,       0,   0,   0,   0,       0,   0,   0,   0,      56,  57,  57,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      59,  62,  64,   0,      60,  61,  61,   0,
       0,   0,   0,   0,       0,   0,   0,   0,      63,  64,  64,   0,       0,   0,   0,   0,
       0,   0,   0,   0
  }
};


const UInt64 TDecPIPETables::sm_Codeword[NUM_V2V_CODERS][65] =
{
  {
           0,       1,      65,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0
  },
  {
           0,       0,     130,      66,       0,       0,      67,     387,       0,       0,       0,     197,       0,       6,    1797,       0,
           0,       0,     198,     326,       0,    1989,     263,       0,     135,     519,       0,       0,       0,     327,       0,     583,
         199,       0,       0,    3782,    3910,       0,    7431,    7687,       0,    7495,    7751,       0,       0,       0,       0,       0,
           0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0
  },
  {
           0,       0,     130,      66,       0,       0,      67,       0,       5,     772,       0,       0,       0,     197,       0,       0,
         964,     262,       0,     134,    1797,       0,       0,     326,    1733,       0,    1861,    3334,       0,       0,     455,       0,
         776,    6791,       0,     840,    6855,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0
  },
  {
           0,       0,     130,      66,       0,       0,      67,       4,       0,       0,     451,     772,       0,       0,     197,     262,
           0,       0,    1733,    3334,       0,       0,     711,    1288,       0,       0,    6855,   13576,       0,       0,    2761,    5386,
           0,       0,   27337,       0,   54538,   10891,       0,   54602,   10955,       0,       0,       0,       0,       0,       0,       0,
           0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0
  },
  {
           0,       0,       0,       5,     194,       0,       0,       0,     133,      69,       0,       0,    1029,     517,       0,     261,
         644,       0,       0,       0,     197,     773,       0,       0,     389,     325,       0,     581,     645,       0,       0,    1093,
        1157,       0,     452,     708,       0,       0,     837,     453,       0,     709,    1221,       0,       0,       0,       0,       0,
           0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0
  },
  {
           0,       2,       0,       0,       0,     517,       0,       0,    1029,     195,       0,     323,     387,       0,       0,     645,
           0,     581,    1093,       0,    1157,     451,       0,     709,    1221,       0,       0,       0,       0,       0,       0,       0,
           0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0
  },
  {
           0,       0,      65,       0,     131,       7,       0,       0,       0,     518,     134,       0,     262,      70,       0,       0,
           0,     388,       0,       0,     136,     389,       0,     325,     197,       0,       0,    1159,     647,       0,     391,     452,
           0,       0,     453,     393,       0,       0,    1223,     711,       0,     455,     457,       0,       0,       0,       0,       0,
           0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0
  },
  {
           0,       3,       0,       0,     131,       0,     259,      67,       0,       0,     772,     388,       0,       0,    5127,       0,
           0,     900,     452,       0,     836,     708,       0,       0,    2630,       0,    5191,    5383,       0,     964,       0,    2758,
        5447,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0
  },
  {
           0,       4,       0,       0,     130,     131,       0,       0,     262,     518,       0,       0,       0,     197,       0,       0,
         325,     389,       0,     773,    1541,       0,    1288,    2312,       0,       0,       0,     451,     452,       0,       0,     836,
         453,       0,     837,    1605,       0,       0,     711,    1223,       0,    1352,    2376,       0,       0,       0,       0,       0,
           0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0
  },
  {
           0,       8,       0,       0,       0,     259,     131,       0,       0,     132,      68,       0,      69,      70,       0,       0,
        1035,    1036,       0,       0,       0,     194,     195,       0,     323,     196,       0,       0,       0,     200,     329,       0,
         201,     586,       0,       0,     330,    1099,       0,     587,    1100,       0,       0,       0,       0,       0,       0,       0,
           0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0
  },
  {
           0,      17,       0,       0,       0,       0,      65,       0,      66,      67,       0,       0,      68,      69,       0,      70,
          71,       0,       0,       0,      72,      73,       0,      74,      75,       0,       0,      76,      77,       0,      78,      79,
           0,      80,      81,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,
           0
  },
  {
           0,      32,       0,       0,       0,       0,       0,      96,      95,       0,      94,      93,       0,       0,      92,      91,
           0,      90,      89,       0,       0,       0,      88,      87,       0,      86,      85,       0,       0,      84,      83,       0,
          82,      81,       0,       0,       0,       0,      80,      79,       0,      78,      77,       0,       0,      76,      75,       0,
          74,      73,       0,       0,       0,      72,      71,       0,      70,      69,       0,       0,      68,      67,       0,      66,
          65
  }
};


